import omc048
import obs
import config
from sensor_drv import SensorDrv

syslog = obs.system_log('LOG')

class AnalogInput(SensorDrv):
    def __init__(self, conf):
        SensorDrv.__init__(self)
        self.raw_value = None
        self.value = None
        self.min_in = config.get_config_params(conf, 'min_in', True)
        self.max_in = config.get_config_params(conf, 'max_in', True)
        self.min_out = config.get_config_params(conf, 'min_out', True)
        self.max_out = config.get_config_params(conf, 'max_out', True)
        self.__name = config.get_config_params(conf, 'log_name', True)
        self.__unit = config.get_config_params(conf, 'log_unit', True)
        self.__tag = config.get_config_params(conf, 'log_tag', True)
    
    async def read_values(self):
        self.raw_value = self.driver.read()
        if isinstance(self.raw_value, float):
            self.value = self.__map(self.raw_value)
            self.data = [self.value]
            return True
        return False

    def detect_parameters(self):
        self.parameters.append((self.__name, self.__unit, self.__tag))
        return True

    def __map(self, adc):
        return ((adc - self.min_in) * (self.max_out - self.min_out) / (self.max_in - self.min_in) + self.min_out)

    async def flush(self):
        #Function override is required for SensorDrv objects
        pass

class VoltInput(AnalogInput):
    def __init__(self, configuration):
        AnalogInput.__init__(self, configuration)
        port = config.get_config_params(configuration, 'port', True)
        self.driver = omc048.analog_voltage(port)

class CurrentInput(AnalogInput):
    def __init__(self, configuration):
        AnalogInput.__init__(self, configuration)
        port = config.get_config_params(configuration, 'port', True)
        self.driver = omc048.analog_current(port)

