#This decoder is used to convert the data of NMEA messages to usable formats.
#If the string inside a field is not used or converted it remains untouched as provided by the original NMEA message
#The output of the decoder is always in list format starting with the original sentence name

from nmea import parser
import obs

__syslog = obs.system_log('NMEA')

PARAM = 0
VALUE = 1

class NmeaDecode(object):
  __nmea_parser = parser()

  #Filter all empty sentence fields ('') data to be converted to 'None'
  def _filter_empty_fields(self, msg):
    for x in msg:
      if x == '':
        msg[msg.index(x)] = None
    return msg

###   GGA   ##################################################################
  def _gga(self, msg):
    if len(msg) < 14:
        return None
    return [((), msg[0]),
            (('UTC', '', 'TIME'), msg[1]),
            (('Latitude', 'deg', 'LAT'), msg[2]),
            (('Dir of lat', '', 'DLAT'), msg[3]),
            (('Longitude', 'deg', 'LON'), msg[4]),
            (('Dir of lon', '', 'DLON'), msg[5]),
            (('Quality', '', 'GPSQ'), msg[6]),
            (('Satellites', '', 'SAT'), msg[7]),
            (('Hdop', 'm', 'HDOP'), msg[8]),
            (('Altitude', 'm', 'ALT'), msg[9]),
            (('Geoid sep', '', 'GEO'), msg[11]),
            (('Age', 'min', 'AGE'), msg[13]),
            (('Station', '', 'STAT'), msg[14])]

###   VTG   ##################################################################
  def _vtg(self, msg):
    if len(msg) < 9:
        return None
    return [((), msg[0]),
            (('Course true', 'deg', 'CRST'), msg[1]),
            (('Course magnetic', 'deg', 'CRSM'), msg[3]),
            (('Speed knots', 'knots', 'SPDK'), msg[5]),
            (('Speed kmh', 'km/h', 'SPDM'), msg[7]),
            (('Mode', '', 'MOD'), msg[9])]

###   RMC   ##################################################################
  def _rmc(self, msg):
    if len(msg) < 12:
        return None
    return [((), msg[0]),
            (('UTC', '', 'TIME'), msg[1]),
            (('Status', '', 'STS'), msg[2]),
            (('Latitude', 'deg', 'LAT'), msg[3]),
            (('Dir of lat', '', 'DLAT'), msg[4]),
            (('Longitude', 'deg', 'LON'), msg[5]),
            (('Dir of lon', '', 'DLON'), msg[6]),
            (('Speed knots', 'knots', 'SPDK'), msg[7]),
            (('Course true', 'deg', 'CRST'), msg[8]),
            (('Date', '', 'DATE'), msg[9]),
            (('Variation', 'deg', 'VAR'), msg[10]),
            (('Mode', '', 'MOD'), msg[11]),
            (('Nav status', '', 'NGST'), msg[12])]

###   TXT   ##################################################################
  def _txt(self, msg):
    if len(msg) < 4:
        return None
    return (((), msg[0]),
            (('Number of sentences', '', 'NOS'), msg[1]),
            (('Sentence number', '', 'SENN'), msg[2]),
            (('Text id', '', 'TXID'), msg[3]),
            (('Message', '', 'MSSG'), msg[4]))

###   PTID  ##################################################################
  def _ptid(self, msg):
    if len(msg) < 9:
        return None
    return (((), msg[0]),
            (('Depth', 'm', 'DEP'), msg[4]),
            (('Status', '', 'STAT'), msg[6]),
            (('Battery voltage', 'V', 'VBAT'), msg[7]),
            (('Barometric pressure', 'hPa', 'BAR'), msg[9]))

  def decode(self, data, checksum=True):
    msg = NmeaDecode.__nmea_parser.parse(data, checksum)
    if msg:
      if msg[0][0] == 'P':
        decoder = self.decoders.get(msg[0])
      else:
        decoder = self.decoders.get(msg[0][2:])
      if decoder is not None:
        return decoder(self,self._filter_empty_fields(msg))
  
###   SUPPORTED DECODERS   ####################################################
  decoders = {  'TXT': _txt, 
                'GGA': _gga,
                'VTG': _vtg,
                'RMC': _rmc,
                'PTID': _ptid  }

